from pathlib import Path
from typing import Optional

import typer
import yaml
from click import Choice
from typing_extensions import Annotated

from eredesscraper.utils import parse_config, flatten_config, struct_config, infer_type
from eredesscraper.workflows import switchboard

appdir = typer.get_app_dir(app_name="ers")
config_path = Path(appdir) / "cache" / "config.yml"

app = typer.Typer(name="ers",
                  help="eredesscraper CLI",
                  add_completion=False,
                  add_help_option=True,
                  no_args_is_help=True,
                  epilog="For more information, please visit https://github.com/rf-santos/eredes-scraper")


@app.command(help="Initialize the program with a CLI wizard")
def init():
    """Initialize the program with a CLI wizard"""
    typer.echo("Welcome to eredesscraper!")
    typer.echo("Please follow the instructions to set up the program.")
    typer.echo("Press Ctrl+C at any time to exit the wizard.")
    typer.echo("")
    typer.echo("First, we need to set up the config file. "
               "See the template `config.yml` at github.com/rf-santos/eredes-scraper for reference.")
    typer.echo("")
    typer.echo("Please enter the path to the config file:")
    config_path = typer.prompt("Path to config file", default="config.yml")
    assert Path(config_path).exists(), "File not found. Please check the path and try again."
    assert Path(config_path).is_file(), "Please enter a valid file path."
    config = parse_config(config_path)
    typer.echo("Config file loaded successfully.")
    typer.echo("")
    typer.echo("Please enter the workflow to run:")
    workflow = typer.prompt("Workflow", default="current_month_consumption", show_choices=True,
                            type=Choice(["current_month_consumption", "last_month_consumption"]))
    typer.echo("")
    typer.echo("Initializing the program...")
    typer.echo("")


@app.command(help="Run the scraper workflow. Can directly load data onto supported databases.")
def run(workflow: str = "current_month_consumption",
        db: Annotated[Optional[str], typer.Argument()] = None):
    """Run a workflow from a config file"""
    config = Path(appdir) / "cache" / "config.yml"
    assert Path(config).exists(), f"Config file not found. Run {typer.style('ers config load </path/to/config.yml>', fg=typer.colors.GREEN)} to load it."

    if db is None:
        db = ""

    switchboard(
        config_path=config.resolve(),
        name=workflow,
        db=db
    )


config_app = typer.Typer(name="config",
                         help="eredesscraper CLI configuration",
                         add_completion=False,
                         add_help_option=True,
                         no_args_is_help=True)

app.add_typer(config_app, name="config", help="Interact with the program configuration")


@config_app.command(help="Loads a config file into the program")
def load(config: str = typer.Argument(resolve_path=True,
                                      exists=True,
                                      file_okay=True,
                                      dir_okay=False,
                                      writable=True,
                                      readable=True,
                                      help="Path to the config file"
                                      )
         ):
    try:
        config_path = Path(config).resolve()

        Path.mkdir(Path(appdir) / "cache", parents=True, exist_ok=True)

        cache = Path(appdir) / "cache"

        typer.echo(f"⚙️\tLoading configuration from file: {config_path}")

        with open(config_path, "r") as f:
            tmp_config = yaml.safe_load(f)

        with open(cache / "config.yml", "w") as f:
            yaml.dump(tmp_config, f)

        typer.echo("✅\tConfig file loaded successfully.")

    except FileNotFoundError:
        typer.echo("💥\tFile not found. Please check the path and try again.")
        raise typer.Exit(code=1)


@config_app.command(help="Show the current configuration")
def show():
    """Show the current configuration"""
    try:
        config = parse_config(Path(appdir) / "cache" / "config.yml")
        typer.echo("")
        typer.echo("⚙️\tCurrent configuration (cached):")
        typer.echo("")
        typer.echo(yaml.dump(config), color=True)
    except FileNotFoundError:
        typer.echo("No config file found. Please run `ers config file /path/to/config.yml` to set a config file.")
        raise typer.Exit(code=1)


@config_app.command(help="Set a configuration value.\nUsage: ers config set <key>[.<subkey>] <value>",
                    options_metavar="<KEY>[.<SUBKEY>] <VALUE>",
                    no_args_is_help=True,
                    epilog="Example: ers config set influxdb.host \"http://localhost\"")
def set(key: str, value: str):
    """
    Set a configuration value

    This command is not idempotent, meaning that it will overwrite the current value of the key in the cached config.

    Infers the type of the value.
    """
    value = infer_type(value)

    try:
        config = flatten_config(parse_config(Path(appdir) / "cache" / "config.yml"))
    except FileNotFoundError:
        typer.echo("No config file found. Please run `ers config file </path/to/config.yml>` to set a config file.")
        raise typer.Exit(code=1)

    assert key in config.keys(), f"Unsupported key {key} not found in config file.\nKeys: {list(config.keys())}"

    config[key] = value

    with open(Path(appdir) / "cache" / "config.yml", "w") as f:
        yaml.dump(struct_config(config), f)

    typer.echo(f"✅\tKey {typer.style(key, fg=typer.colors.GREEN)} set to {typer.style(value, fg=typer.colors.GREEN)}.")


if __name__ == "__main__":
    app()



import datetime
from pathlib import Path

import influxdb_client
import pandas as pd
import typer
from influxdb_client import WritePrecision
from influxdb_client.client.exceptions import InfluxDBError
from influxdb_client.client.write_api import SYNCHRONOUS

from eredesscraper.utils import parse_monthly_consumptions


class InfluxDB:
    def __init__(self,
                 token,
                 org,
                 bucket,
                 host: str = "http://localhost",
                 port: int = 8086):
        self.__bucket = bucket
        self.__host = host
        self.__port = port
        self.__url = str(self.__host) + ':' + str(self.__port)
        self.__token = token
        self.__org = org
        self.client = None
        self.last_insert = None
        self.data = None
        self.debug = False

    # define a setter method for the host property
    @property
    def host(self):
        return self.__host

    @host.setter
    def host(self, value):
        if isinstance(value, str):
            self.__host = value
        else:
            raise TypeError("Expected a string value")

    # define a setter method for the port property
    @property
    def port(self):
        return self.__port

    @port.setter
    def port(self, value):
        if isinstance(value, str):
            self.__port = value
        else:
            raise TypeError("Expected a string value")

    # define a setter method for the token property
    @property
    def token(self):
        return self.__token

    @token.setter
    def token(self, value):
        if isinstance(value, str):
            self.__token = value
        else:
            raise TypeError("Expected a string value")

    # define a setter method for the org property
    @property
    def org(self):
        return self.__org

    @org.setter
    def org(self, value):
        if isinstance(value, str):
            self.__org = value
        else:
            raise TypeError("Expected a string value")

    def connect(self) -> None:
        """
        The ``connect`` method connects to the InfluxDB database using the specified host, port, token and org.

        :return: None
        :doc-author: Ricardo Filipe dos Santos
        """
        try:
            self.client = influxdb_client.InfluxDBClient(url=self.__url,
                                                         token=self.__token,
                                                         org=self.__org,
                                                         debug=self.debug,
                                                         enable_gzip=False)
        except InfluxDBError:
            typer.echo("💥\tError connecting to the InfluxDB database")
        finally:
            typer.echo(f"🔗\tConnected to the InfluxDB database at {self.__url}")

        return None

    def load(self, source_data: Path, cpe_code) -> None:
        """
        The ``load`` method loads the data parsed as a pandas DataFrame into the InfluxDB database.

        :param source_data: Specify the path to the source data .XLSX file to be loaded
        :return: None
        :doc-author: Ricardo Filipe dos Santos
        """

        self.client.write_api(write_options=SYNCHRONOUS).write(bucket=self.__bucket,
                                                               org=self.__org,
                                                               record=self.delta(source_data, cpe_code=cpe_code),
                                                               data_frame_measurement_name='kW',
                                                               data_frame_tag_columns=['cpe'],
                                                               data_frame_field_columns=['consumption'],
                                                               data_frame_timestamp='date_time',
                                                               data_frame_write_precision=WritePrecision.S)

        self.client.close()

        # remove the source_data .XLSX file and staging area
        try:
            source_data.unlink()
            Path.rmdir(Path.cwd() / 'tmp')
            typer.echo(f"💀\tRemoved the source data file and the staging area: {source_data}")
        except PermissionError:
            typer.echo("💥\tPermission denied to remove the staging area for the Scraper")

        typer.echo(f"✅\tLoaded data from {source_data} into the InfluxDB database")

        return None

    def get_last_insert(self, cpe_code: str) -> datetime:
        """
        The ``get_last_insert`` method returns the ``datetime`` object representing the latest data point present in
        the InfluxDB database. The query filters the data points by the CPE code.

        :param cpe_code: Specify the CPE code of the data point to be returned
        :return: A Datetime object with the latest data point timestamp
        :doc-author: Ricardo Filipe dos Santos
        """

        query = (f'''import "date"
from(bucket: "{self.__bucket}")
  |> range(start: 1989-05-11T01:10:00Z, stop: now())
  |> filter(fn: (r) => r["_measurement"] == "kW")
  |> filter(fn: (r) => r["_field"] == "consumption")
  |> filter(fn: (r) => r["cpe"] == "{cpe_code}")
  |> last() |> pivot(rowKey:["_time"], columnKey: ["_field"], valueColumn: "_value")''')

        result = self.client.query_api().query_data_frame(org=self.__org, query=query)

        # test if result is empty
        if result.empty:
            return datetime.datetime(
                1989,
                5,
                11,
                0,
                0,
                0,
                tzinfo=datetime.timezone(datetime.timedelta(hours=1))
            )
        else:
            return result['_time'].iloc[0].dt.tz_localize('Europe/Lisbon')

    def delta(self, source_data: Path, cpe_code: str) -> pd.DataFrame:
        """
        The ``delta`` method uses ``parse_monthly_consumptions`` function to get the source data into a pandas
        DataFrame and keep only the values that are more recent than the result of the ``get_last_insert`` method.
        The method returns the filtered DataFrame holding the most recent data not present in the DB bucket.

        :param source_data: Specify the source data to be filtered
        :param cpe_code: Specify the CPE code of the data point to be returned
        :return: A pandas DataFrame with the filtered data
        :doc-author: Ricardo Filipe dos Santos
        """

        # get the source data into a pandas DataFrame
        df = parse_monthly_consumptions(source_data, cpe_code=cpe_code)

        # filter out the data points that are already present in the DB bucket
        df = df[df.index > self.get_last_insert(cpe_code=cpe_code)]

        self.data = df
        return df



# package imports
from pathlib import Path

import typer

from eredesscraper.agent import EredesScraper
from eredesscraper.db_clients import InfluxDB
from eredesscraper.utils import parse_config


def switchboard(name: str, db: str, config_path: Path) -> None:
    """
    The run function is the entry point.

    :param name: str: Specify which workflow to run. One of: ``current_month_consumption``
    :type name: str
    :param db: str: Specify which database to use. One of: ``influxdb``
    :type db: str
    :param config_path: Path: Specify the path to the config file
    :type config_path: pathlib.Path
    :return: None
    :doc-author: Ricardo Filipe dos Santos
    """

    config = parse_config(config_path=config_path)

    match name:
        case 'current_month_consumption':
            typer.echo(f"🚀\tRunning {typer.style('current_month_consumption', fg=typer.colors.GREEN)} workflow")

            typer.echo(f"📇\tE-REDES client info: "
                       f"NIF: {typer.style(config['eredes']['nif'], fg=typer.colors.GREEN, bold=True)}, "
                       f"CPE: {typer.style(config['eredes']['cpe'], fg=typer.colors.GREEN, bold=True)}")

            bot = EredesScraper(
                nif=config['eredes']['nif'],
                password=config['eredes']['pwd'],
                cpe_code=config['eredes']['cpe']
            )
            bot.setup()
            bot.current_month_consumption()
            bot.teardown()

        case _:
            typer.echo(f"??\tWorkflow {typer.style(name, fg=typer.colors.GREEN)} not supported")

    match db:
        case 'influxdb':
            db = InfluxDB(
                token=config['influxdb']['token'],
                org=config['influxdb']['org'],
                host=config['influxdb']['host'],
                port=config['influxdb']['port'],
                bucket=config['influxdb']['bucket'])
            db.connect()
            db.load(source_data=bot.dwnl_file, cpe_code=config['eredes']['cpe'])

        case '':
            pass

        case _:
            typer.echo(f"??\tDatabase {typer.style(db, fg=typer.colors.GREEN)} not supported")


import os
import time
from collections.abc import MutableMapping
from pathlib import Path
from typing import Union

import pandas as pd
import yaml
from selenium import webdriver
from selenium.webdriver.common.by import By


def parse_monthly_consumptions(file_path: Path, cpe_code: str) -> pd.DataFrame:
    """
    The parse_file function takes a XLSX file path retrieved from E-REDES and returns
    a pandas DataFrame with the parsed data.
    An example for the retrieved file can be found in the `tests` folder.
    TZ is set to Europe/Lisbon
    Parsing rules:
    - The first 7 lines are skipped
    - The table has 3 columns: Date, Time and Value
    - The Date is "date", the Time is "time", the Value is "consumption"
    - The "date" and "time" columns are merged into a single column named "date_time"

    :param file_path: Specify the Excel (.XLSX) file path of the file to be parsed
    :type file_path: pathlib.Path
    :param cpe_code: Specify the CPE code to be added to the DataFrame
    :type cpe_code: str
    :return: A pandas DataFrame with the parsed data
    :doc-author: Ricardo Filipe dos Santos
    """

    df = pd.read_excel(
        file_path,
        skiprows=7,
        parse_dates=[[0, 1]],
        names=['date', 'time', 'consumption'],
        dtype={'consumption': float},
        decimal=',',
        thousands='.'
    )

    # add the cpe code from the config file to all rows
    df['cpe'] = cpe_code

    # add the date_time column
    df['date_time'] = df['date_time'].dt.tz_localize('Europe/Lisbon')
    df.set_index('date_time', inplace=True)

    return df


def flatten_config(d, parent_key='', sep='.') -> dict:
    """
    The flatten_config function takes a dictionary and flattens it into a single level.
    For example, if the input is:
    {'a': 1, 'b': {'x': 2, 'y': 3}, 'c': 4}
    then the output will be:
    {'a': 1, 'b.x': 2, 'b.y', 3 ,'c', 4}

    :param d: Pass the dictionary to be flattened
    :type d: dict
    :param parent_key: Keep track of the parent key
    :type parent_key: str
    :param sep: Separate the keys in the nested dictionary
    :type sep: str
    :return: A dictionary with all the keys and values from a nested dictionary
    :doc-author: Ricardo Filipe dos Santos
    """
    items = []
    for k, v in d.items():
        new_key = parent_key + sep + k if parent_key else k
        if isinstance(v, MutableMapping):
            items.extend(flatten_config(v, new_key.lower(), sep=sep).items())
        else:
            items.append((new_key.lower(), v))
    return dict(items)


def flatten_keys(d: dict, sep: str = '.', keep_level: int = -1) -> dict:
    """
    The flatten_keys function takes a dictionary and returns a new dictionary with the keys
    split by the specified separator.
    The keep_level parameter specifies which list index to keep after key.split(),
    by default keeps last item of split.

    :param d: Specify the dictionary that is to be converted
    :type d: dict
    :param sep: Split the key string into a list of substrings
    :type sep: str
    :param keep_level: Keep only the `keep_level` index of the split key name
    :type keep_level: int
    :return: A dictionary with the modified keys of `d`
    :doc-author: Ricardo Filipe dos Santos
    """
    items = {}
    for k, v in d.items():
        items[k.split(sep)[keep_level]] = v

    return items


def struct_config(d: dict, sep: str = ".") -> dict:
    """
    The struct_config function takes a dictionary as the one given by the flatten_config function and structures it
    with nested dicts, useful to dump as YAML file.

    :param d: Specify the dictionary that is to be converted. (typically, a flattened dictionary)
    :type d: dict
    :param sep: Specify the separator used in the flattened dictionary.
    :type sep: str
    :return: A dictionary with the modified keys of `d`
    :doc-author: Ricardo Filipe dos Santos
    """
    items = {}
    for k, v in d.items():
        keys = k.split(sep)
        if len(keys) == 1:
            items[k] = v
        else:
            if keys[0] not in items:
                items[keys[0]] = {}
            items[keys[0]][keys[1]] = v

    return items


def parse_config(config_path: Path = Path.cwd() / "config.yml") -> dict:
    """
    The parse_config function parses the config.yml file and returns a dictionary with the parsed data.

    :param config_path: Specify the path to the config.yml file
    :type config_path: pathlib.Path
    :return: A dictionary with the parsed data from the config.yml file
    :doc-author: Ricardo Filipe dos Santos
    """
    with open(config_path, "r") as f:
        config = yaml.safe_load(f)

    return config


def save_screenshot(driver: webdriver.Chrome, path: str = 'screenshot.png') -> None:
    # Ref: https://stackoverflow.com/a/52572919/
    original_size = driver.get_window_size()
    required_width = driver.execute_script('return document.body.parentNode.scrollWidth')
    required_height = driver.execute_script('return document.body.parentNode.scrollHeight')
    driver.set_window_size(required_width, required_height)
    # driver.save_screenshot(path)  # has scrollbar
    driver.find_element(By.TAG_NAME, "body").screenshot(path)  # avoids scrollbar
    driver.set_window_size(original_size['width'], original_size['height'])


def wait_for_download(directory, timeout, nfiles=None):
    seconds = 0
    dl_wait = True
    while dl_wait and seconds < timeout:
        time.sleep(1)
        dl_wait = False
        files = os.listdir(directory)
        if nfiles and len(files) != nfiles:
            dl_wait = True

        for fname in files:
            if fname.endswith('.crdownload'):
                dl_wait = True

        seconds += 1
    return seconds


def config2env(flat_config: dict):
    """
    The config2env function takes a dictionary and converts it to a string in the form of
    `key=value` pairs, separated by a newline character. This is then exported to the environment
    variables.

    :param flat_config: Specify the dictionary to be converted
    :type flat_config: dict
    :return: None
    :doc-author: Ricardo Filipe dos Santos
    """
    for k, v in flat_config.items():
        os.environ[k.upper()] = str(v)


def infer_type(value: str) -> Union[str, int, float, bool]:
    if value.isnumeric():
        return int(value)
    elif value.lower() in ["True", "False", "true", "false", "yes", "no", "y", "n", "1", "0"]:
        return bool(value)
    elif value.replace(".", "", 1).isnumeric():
        return float(value)
    else:
        return value



import datetime
import os
import time
from pathlib import Path
from sys import platform

import typer
from rich.progress import Progress, SpinnerColumn, TextColumn
from selenium import webdriver
from selenium.webdriver.chrome.service import Service as ChromeService
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait as wait
from webdriver_manager.chrome import ChromeDriverManager

from eredesscraper.utils import wait_for_download, save_screenshot


class ScraperFlowError(Exception):
    pass


class EredesScraper:
    def __init__(self, nif, password, cpe_code):
        self.dwnl_file = None
        self.driver = None
        self.chrome_options = webdriver.ChromeOptions()
        self.headless = True
        self.__nif = nif
        self.__password = password
        self.__cpe_code = cpe_code

        # Create a staging area for the ChromeDriver
        try:
            Path.mkdir(Path.cwd() / 'tmp', exist_ok=True)
        except PermissionError:
            print("Permission denied to create a staging area for the Scraper")

        dl_path = Path.cwd() / 'tmp'
        if platform == "linux" or platform == "linux2" or platform == "darwin":
            self.tmp = dl_path.as_posix()
        elif platform == "win32":
            self.tmp = dl_path.__str__()

    # define a setter method for the headless property
    @property
    def headless(self):
        return self.__headless

    @headless.setter
    def headless(self, value):
        if isinstance(value, bool):
            self.__headless = value
        else:
            raise TypeError("Expected a boolean value")

    # define a setter method for the nif property
    @property
    def nif(self):
        return self.__nif

    @nif.setter
    def nif(self, value):
        if isinstance(value, str):
            self.__nif = value
        else:
            raise TypeError("Expected a string value")

    # define a setter method for the password property
    @property
    def password(self):
        return self.__password

    @password.setter
    def password(self, value):
        if isinstance(value, str):
            self.__password = value
        else:
            raise TypeError("Expected a string value")

    # define a setter method for the cpe_code property
    @property
    def cpe_code(self):
        return self.__cpe_code

    @cpe_code.setter
    def cpe_code(self, value):
        if isinstance(value, str):
            self.__cpe_code = value
        else:
            raise TypeError("Expected a string value")

    def setup(self):
        self.chrome_options.add_argument('user-agent=Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 '
                                         '(KHTML, like Gecko) Chrome/84.0.4147.125 Safari/537.36')

        prefs = {
            "download.default_directory": self.tmp,
            "credentials_enable_service": False,
            "profile.password_manager_enabled": False
        }
        self.chrome_options.add_experimental_option("prefs", prefs)
        self.chrome_options.add_experimental_option("excludeSwitches", ["enable-automation"])
        self.chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])
        self.chrome_options.add_experimental_option('useAutomationExtension', False)
        if self.headless:
            self.chrome_options.add_argument("--headless=new")
            self.chrome_options.add_argument('--disable-blink-features=AutomationControlled')
            self.chrome_options.add_argument('--disable-infobars')
            self.chrome_options.add_argument('--disable-dev-shm-usage')
            self.chrome_options.add_argument('--disable-browser-side-navigation')
            self.chrome_options.add_argument('--disable-gpu')
            self.chrome_options.add_argument('--disable-features=VizDisplayCompositor')
            self.chrome_options.add_argument('--disable-extensions')
            self.chrome_options.add_argument('--no-sandbox')
            self.chrome_options.add_argument('--disable-notifications')
            self.chrome_options.add_argument('--disable-crash-reporter')
            self.chrome_options.add_argument('--disable-logging')
            self.chrome_options.add_argument('--disable-sync')

        self.driver = webdriver.Chrome(
            service=ChromeService(ChromeDriverManager().install()),
            options=self.chrome_options
        )

        self.driver.implicitly_wait(30)

    def teardown(self):
        self.driver.quit()

    def current_month_consumption(self):
        with Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                transient=True,
        ) as progress:
            # Selenium flow
            # Step # | name | target | value | comment

            t1 = progress.add_task(description=" 🔐 Loging in...", total=None)
            # 1 | open | /login |  |
            self.driver.get("https://balcaodigital.e-redes.pt/login")

            # 2 | click | css=.ant-typography > .item > .highlights |  |
            wait(self.driver, 30).until(EC.presence_of_element_located(
                (By.CSS_SELECTOR, ".ant-typography > .item > .highlights")
            )).click()

            # 3 | type | id=username |   |
            self.driver.find_element(By.ID, "username").send_keys(f"{self.__nif}")

            # 4 | type | id=labelPassword |  |
            self.driver.find_element(By.ID, "labelPassword").send_keys(f"{self.__password}")

            # 5 | click | css=.login-actions |  |
            self.driver.find_element(By.XPATH, "//span[contains(.,'Entrar')]").click()
            time.sleep(5)

            # 7 | click | css=.card__myplaces .card-text |  |
            wait(self.driver, 30).until(EC.presence_of_element_located(
                (By.CSS_SELECTOR, ".card__myplaces .card-text")
            )).click()

            progress.remove_task(t1)
            t2 = progress.add_task(description=" 💡 Finding your CPE...", total=None)
            # 8 | click | css=.card:nth-child(3) .highlights |  |
            wait(self.driver, 30).until(EC.presence_of_element_located(
                (By.CSS_SELECTOR, ".card:nth-child(3) .highlights")
            )).click()

            # 9 | click | css=.card:nth-child(1) > .item__title |  |
            wait(self.driver, 30).until(EC.presence_of_element_located(
                (By.CSS_SELECTOR, ".card:nth-child(1) > .item__title")
            )).click()

            # 10 | click | css=.list:nth-child(2) > .block:nth-child(1) .card-tags |  |
            wait(self.driver, 30).until(EC.presence_of_element_located(
                (By.XPATH, f"//*[contains(text(),'{self.__cpe_code}')]")
            )).click()

            progress.remove_task(t2)
            t3 = progress.add_task(description=" 📊 Downloading your data...", total=None)
            time.sleep(30)
            try:
                element = self.driver.find_element(By.XPATH, "//strong[contains(text(), 'Exportar excel')]")
            except:
                save_screenshot(self.driver, "screenshot.png")
                raise ScraperFlowError("Failed to find the 'Exportar excel' element")

            self.driver.execute_script("arguments[0].scrollIntoView();", element)
            element.click()

            wait_for_download(self.tmp, 30)

            current_date = datetime.datetime.now().strftime("%Y%m%d")
            assert f"Consumos_{current_date}.xlsx" in os.listdir(self.tmp), "Download failed"

            # store the downloaded file absolute path in a variable
            self.dwnl_file = Path(self.tmp) / f"Consumos_{current_date}.xlsx"

        progress.remove_task(t3)
        typer.echo(f"📁 Downloaded file: {self.dwnl_file}")
        
        return self.dwnl_file
        



from os.path import dirname, isdir, join
import os
import re
import subprocess
from pathlib import Path

version_re = re.compile('^Version: (.+)$', re.M)


def get_version():
    # This program is placed into the public domain.

    """
    Gets the current version number.
    If in a git repository, it is the current git tag.
    Otherwise, it is the one contained in the PKG-INFO file.
    To use this script, simply import it in your setup.py file
    and use the results of get_version() as your package version:
        from version import *
        setup(
            ...
            version=get_version(),
            ...
        )
    """

    global version

    # Paths for package structure
    d = Path(dirname(__file__))
    src = d.parent
    root = d.parent

    try:
        pkg = [p for p in src.iterdir() if p.is_dir() and 'egg-info' in p.name]
    except ValueError:
        pkg = None
        pass

    if isdir(join(root, '.git')):
        # Get the version using "git describe".
        cmd = 'git describe --tags --match [0-9]*'.split()
        try:
            version = subprocess.check_output(cmd).decode().strip()
        except subprocess.CalledProcessError:
            print('Unable to get version number from git tags')
            exit(1)

        # PEP 386 compatibility
        if '-' in version:
            version = '.post'.join(version.split('-')[:2])

        # Don't declare a version "dirty" merely because a time stamp has
        # changed. If it is dirty, append a ".dev1" suffix to indicate a
        # development revision after the release.
        with open(os.devnull, 'w') as fd_devnull:
            subprocess.call(['git', 'status'],
                            stdout=fd_devnull, stderr=fd_devnull)

        cmd = 'git diff-index --name-only HEAD'.split()
        try:
            dirty = subprocess.check_output(cmd).decode().strip()
        except subprocess.CalledProcessError:
            print('Unable to get git index status')
            exit(1)

        if dirty != '':
            version += '.dev1'

    elif pkg:
        try:
            # Extract the version from the PKG-INFO file.
            with open(join(pkg[0], 'PKG-INFO')) as f:
                version = version_re.search(f.read()).group(1)
        except (IOError, AttributeError):
            print('Unable to find version number in PKG-INFO file')
            exit(1)

    # Fallback to a hard-coded version number of installed package
    else:
        output = subprocess.check_output(['pip', 'show', 'eredesscraper'])
        for elem in output.decode("utf-8").replace("\r", "").split("\n"):
            if "version" in elem.lower():
                version = elem.split(":")[-1].strip()
                break
    return version


if __name__ == '__main__':
    print(get_version())